#include "diseqc.h"
#include <sys/ioctl.h>
#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/select.h>
#include <libio.h>

#define SWITCHFREQ 11700000
#define LOF_HI     10600000
#define LOF_LO      9750000

#define FE "/dev/dvb/adapter0/frontend0"
#define DX "/dev/dvb/adapter0/demux0"

#define ERROR(txt) printf("%s: %s\n", txt, strerror(errno))

int activeFreq;
int frontEnd;

void setFreq(struct dvb_frontend_parameters *freq, int *polarisation)
{
  if(activeFreq==0) {
    freq->frequency=11931000;
    freq->u.qpsk.symbol_rate=27500000;
    freq->u.qpsk.fec_inner=FEC_3_4;
    *polarisation=1;
    activeFreq=1;
  } else {
    freq->frequency=11771000;
    freq->u.qpsk.symbol_rate=27500000;
    freq->u.qpsk.fec_inner=FEC_3_4;
    *polarisation=1;
    activeFreq=0;
  }

    freq->inversion=INVERSION_AUTO;
}



void readStuff(int demux)
{
  fd_set rfds;
  struct timeval tv;
  int retval;
  char buffer[4096];

  FD_ZERO(&rfds);
  FD_SET(demux, &rfds);

  tv.tv_sec = 20;
  tv.tv_usec = 0;


  retval = select(demux+1, &rfds, NULL, NULL, &tv);

  if (retval == -1)
    ERROR("select");

  else if (retval) {
    if(read(demux,buffer,4096)<=0) {
      ERROR("Cannot read");
    }
    if(buffer[0]!=0x00) {
      printf("Got some trash\n");
      exit(-1);
    }
    printf("Data is available.\n");

  } else {
    printf("No data within twenty seconds. Reseting the ARM\n");
    if(ioctl(frontEnd,74,NULL)==-1) {
      ERROR("Cannot reset the ARM");
      exit(-1);
    }
  }

}



int main(int argc, char **argv)
{
  int demux;
  int hiband;
  int ret;
  struct dvb_frontend_parameters params;
  struct dmx_sct_filter_params sctFilter;
  int polarisation;
  int iteration=0;
    
  memset(sctFilter.filter.filter,0,DMX_FILTER_SIZE);
  memset(sctFilter.filter.mask,0,DMX_FILTER_SIZE);
  sctFilter.pid=0x0000;
  sctFilter.timeout=0;
  //sctFilter.filter.filter[0]=0x00;
  //sctFilter.filter.mask[0]=0xFF;
  sctFilter.flags=DMX_IMMEDIATE_START|DMX_CHECK_CRC;

  activeFreq=0;

  while(1) {
    frontEnd=open(FE,O_RDWR);
    if(frontEnd<0) {
      ERROR("Cannot open the frontend");
      exit(-1);
    }

    printf("Iteration %d==========================",++iteration);
    setFreq(&params, &polarisation);
    printf("Tune to %u...\n",params.frequency);

    hiband=((params.frequency>=SWITCHFREQ)?1:0);
    if(hiband) {
      params.frequency-=LOF_HI;
    } else {
      params.frequency-=LOF_LO;
    }
    if((ret=setup_switch(frontEnd,0,
                         polarisation==1?0:1,hiband))!=0) {
      ERROR("Cannot setup switch");
    }
  
    if(ioctl(frontEnd, FE_SET_FRONTEND, &params)==-1) {
      ERROR("Cannot tune");
      exit(-1);
    }

    demux=open(DX,O_RDWR);
    if(demux>=0) {
      if(ioctl(demux,DMX_SET_BUFFER_SIZE,2048)==-1) {
        ERROR("Cannot setup the buffer size");
        exit(-1);
      }

      if(ioctl(demux,DMX_SET_FILTER,&sctFilter)==-1) {
        ERROR("Cannot set the filter");
        exit(-1);
      }

      printf("read...\n");
      readStuff(demux);

      ioctl(demux,DMX_STOP);
      close(demux);

      printf("done...\n");
      
    } else {
      ERROR("Cannot open the demux");
      exit(-1);
    }

    close(frontEnd);
    sleep(10);
  }
}
